<?php

namespace App\Helpers;

/**
 * Description of VideoStream
 *
 * @author Rana
 * modified by HazCod to use stream_get_contents and correct session shutoff
 * https://github.com/HazCod
 * @link http://codesamplez.com/programming/php-html5-video-streaming-tutorial
 **/
class VideoStream
{
	private string $path;
	private mixed $stream;
	private int $buffer = 102400;
	private int $start = -1;
	private int $end = -1;
	private int $size = 0;

	function __construct(string $filename)
	{
		$this->path = $filename;
	}

	/**
	 * Open stream
	 **/
	private function open()
	{
		if (! ($this->stream=fopen($this->path,'rb',false, stream_context_create())))
			die('Could not open stream for reading');
	}

	/**
	 * Set proper header to serve the video content
	 **/
	private function setHeader()
	{
		ob_get_clean();
		header('Content-Type: '.mime_content_type($this->path));
		header(sprintf('Cache-Control: max-age=%d, public',$x=60*60*24*30));
		header(sprintf('Expires: %s GMT',gmdate('D, d M Y H:i:s',time()+$x)));
		header(sprintf('Last-Modified: %s GMT',gmdate('D, d M Y H:i:s',filemtime($this->path))));

		$this->start = 0;
		$this->size = filesize($this->path);
		$this->end = $this->size - 1;
		header('Accept-Ranges: 0-'.$this->end);

		if (isset($_SERVER['HTTP_RANGE'])) {
			$c_end = $this->end;

			list(,$range) = explode('=',$_SERVER['HTTP_RANGE'],2);
			if (strpos($range,',') !== FALSE) {
				header('HTTP/1.1 416 Requested Range Not Satisfiable');
				header(sprintf('Content-Range: bytes %d-%d/%d',$this->start,$this->end,$this->size));
				exit;
			}

			if ($range == '-') {
				$c_start = $this->size - substr($range,1);

			} else {
				$range = explode('-',$range);
				$c_start = $range[0];
				$c_end = (isset($range[1]) && is_numeric($range[1])) ? $range[1] : $c_end;
			}

			$c_end = ($c_end > $this->end) ? $this->end : $c_end;

			if ($c_start > $c_end || $c_start > $this->size - 1 || $c_end >= $this->size) {
				header('HTTP/1.1 416 Requested Range Not Satisfiable');
				header(sprintf('Content-Range: bytes %d-%d/%d',$this->start,$this->end,$this->size));
				exit;
			}

			$this->start = $c_start;
			$this->end = $c_end;
			$length = $this->end - $this->start + 1;

			fseek($this->stream,$this->start);
			header('HTTP/1.1 206 Partial Content');
			header("Content-Length: ".$length);
			header(sprintf('Content-Range: bytes %d-%d/%d',$this->start,$this->end,$this->size));

		} else {
			header("Content-Length: ".$this->size);
		}
	}

	/**
	 * close curretly opened stream
	 **/
	private function end()
	{
		fclose($this->stream);
		exit;
	}

	/**
	 * perform the streaming of calculated range
	 **/
	private function stream()
	{
		$i = $this->start;
		set_time_limit(0);
		while ((! feof($this->stream)) && ($i <= $this->end) && (connection_aborted() == 0)) {
			$bytesToRead = $this->buffer;

			if (($i+$bytesToRead) > $this->end) {
				$bytesToRead = $this->end - $i + 1;
			}

			echo stream_get_contents($this->stream,$bytesToRead);
			flush();
			$i += $bytesToRead;
		}
	}

	/**
	 * Start streaming video content
	 **/
	function start()
	{
		session_write_close();
		$this->open();
		$this->setHeader();
		$this->stream();
		$this->end();
	}
}